/*
 * Copyright (c) Markus Sonderegger  <msonderegger@aon.at>
 * This copyright notice must be included in all copies 
 * or substantial portions of the Software. 
 */
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <X11/xpm.h>

static Display *dpy;
static Window root;
static int screen = 0;

static void
eprint(const char *errstr, ...) {
    va_list ap;

    va_start(ap, errstr);
    vfprintf(stderr, errstr, ap);
    va_end(ap);
    exit(1);
}

static void
setxpm(char *file) {
    int val;
    XpmAttributes xa;
    XWindowAttributes wa;
    Pixmap pixmap, shape;

    XGetWindowAttributes(dpy, root, &wa);
    xa.colormap = wa.colormap;
    xa.valuemask = XpmSize | XpmReturnPixels | XpmColormap;

    if((val =
        XpmReadFileToPixmap(dpy, root, file, &pixmap, &shape,
                            &xa)) != XpmSuccess) {
        if(val == XpmOpenFailed)
            eprint("sroot: cannot open pixmap file\n");
        else if(val == XpmColorFailed)
            eprint("sroot: cannot allocat required colors\n");
        else if(val == XpmFileInvalid)
            eprint("sroot: invalid format for an xpm file\n");
        else if(val == XpmColorError)
            eprint("sroot: invalid color specified in xpm file\n");
        else if(val == XpmNoMemory)
            eprint("sroot: insufficient memory\n");
    }
    XSetWindowBackgroundPixmap(dpy, root, pixmap);

    XClearWindow(dpy, root);
    XSetCloseDownMode(dpy, RetainPermanent);
    XCloseDisplay(dpy);
}

static void
setbg(const char *colstr) {
    XColor color;
    Colormap cmap = DefaultColormap(dpy, screen);
    XAllocNamedColor(dpy, cmap, colstr, &color, &color);

    unsigned long bg = color.pixel;
    XSetWindowBackground(dpy, root, bg);

    XClearWindow(dpy, root);
    XSetCloseDownMode(dpy, RetainPermanent);
    XCloseDisplay(dpy);
}

int
main(int argc, char *argv[]) {
    dpy = XOpenDisplay(NULL);
    if(!dpy)
        eprint("sroot: cannot open display\n");
    root = RootWindow(dpy, screen);

    if(argc == 3 && !strncmp("-xpm", argv[1], 5))
        setxpm(argv[2]);
    else if(argc == 3 && !strncmp("-bg", argv[1], 4))
        setbg(argv[2]);
    else
        eprint("usage: sroot [-xpm <xpm-image>] [-bg <color>]\n");

    return 0;
}
